import logging
import datetime
import io
import conf
import os

if conf.CLOUD_PROVIDER != conf.MOCK and conf.CLOUD_PROVIDER != conf.LOCAL:
    from bundles import storage


logger = logging.getLogger(__name__)


def audit_filename(data_partition):
    date_string = datetime.datetime.now().strftime("%Y%m%d-%H%M%S")
    logger.debug(f"partition {data_partition} bundle")
    return f"audit-{data_partition}-{date_string}.log"


def write_audit_log(data_partition, log_message):
    cloud_provider = os.environ.get("CLOUD_PROVIDER")
    if (
        cloud_provider is None
        or cloud_provider == conf.MOCK
        or cloud_provider == conf.LOCAL
    ):
        return

    if conf.ENABLE_EXTERNAL_AUDIT_LOG:
        filename = audit_filename(data_partition)

        with io.BytesIO() as f:
            f.write(log_message.encode("utf-8"))
            f.seek(0)
            # print(f.read())
            if storage.get_storage().upload_file(filename, f):
                logger.info(f"Submitted audit log {filename}")
            else:
                logger.error(f"Error storing audit log {filename}")
