import os
from dotenv import load_dotenv

load_dotenv()  # take environment variables from .env.

#  ENTITLEMENTS_BASE_URL is the https/http endpoint for entitlement service
ENTITLEMENTS_GROUPS_API = os.getenv(
    "ENTITLEMENTS_BASE_URL", "http://entitlements"
) + os.getenv("ENTITLEMENTS_BASE_PATH", "/api/entitlements/v2/groups")
LEGAL_BATCH_API = (
    os.getenv("LEGAL_BASE_URL", "http://legal-service")
    + "/api/legal/v1/legaltags:batchRetrieve"
)
LEGAL_BASE_URL = os.getenv("LEGAL_BASE_URL")
# Partition API is only used in testing
PARTITION_API = (
    os.getenv("PARTITION_BASE_URL", "http://partition") + "/api/partition/v1"
)

# Diagnostics should never be enabled in true production environments
ENABLE_DEV_DIAGNOSTICS = os.getenv("ENABLE_DEV_DIAGNOSTICS", "False").lower() in (
    "true",
    "1",
    "t",
)

# Diagnostics should never be enabled in true production environments
# Create DUMP outputs - for troubleshooting eval, etc
ENABLE_DEV_DIAGNOSTICS_DUMPS = os.getenv(
    "ENABLE_DEV_DIAGNOSTICS_DUMPS", "False"
).lower() in ("true", "1", "t")

localhost_opa_url = "http://localhost:8181"
OPA_URL = os.getenv("OPA_URL", localhost_opa_url)
if len(OPA_URL) == 0:
    OPA_URL = localhost_opa_url

# OPA APIs
OPA_POLICIES_API = OPA_URL + "/v1/policies"
OPA_DATA_API = OPA_URL + "/v1/data"
OPA_COMPILE_API = OPA_URL + "/v1/compile"
OPA_CONFIG_API = OPA_URL + "/v1/config"
OPA_HEALTH_API = OPA_URL + "/health"
OPA_STATUS_API = OPA_URL + "/v1/status"

USE_BUNDLES = os.getenv("USE_BUNDLES", True)

# Require users and admin to be in these groups
USER_PERMISSION = "service.policy.user"
ADMIN_PERMISSION = "service.policy.admin"
OPS_PERMISSION = os.getenv("OPS_PERMISSION", ADMIN_PERMISSION).lower()

# Search Translate PreProcessor
# It is highly recommended to always use the translate preprocessor.
# Without it policies are restricted to simple policies only (for example no deny or other complex logic)
# Included here for diagnostic, troubleshooting and testing purposes only
# If disabling this care must be given to make sure only simple policies are used.
ENABLE_TRANSLATE_PREPROCESSOR = os.getenv(
    "ENABLE_TRANSLATE_PREPROCESSOR", "True"
).lower() in ("true", "1", "t")

# Enable External Audit Log
ENABLE_EXTERNAL_AUDIT_LOG = os.getenv("ENABLE_EXTERNAL_AUDIT_LOG", "True").lower() in (
    "true",
    "1",
    "t",
)

# OPA bug in OPA Version: 0.43.0 where policy IDs are get a leading /
OPA_BUG_FORCE_SLASH = False

# Prefix for all policy service APIs
SERVICE_BASE_PATH = "/api/policy/v1"

INSTANCE_BUNDLE_ROOT = "osdu/instance"
PARTITION_BUNDLE_ROOT = "osdu/partition/{0}"

# support local testing and mock
LOCAL = "LOCAL"
MOCK = "MOCK"

# Generate a Correlation ID when one is not provided in request
GENERATE_CORRELATION_ID = True

# LOGLEVEL
LOG_LEVEL = os.getenv("LOG_LEVEL", "INFO").upper()
LOG_LEVEL_TRANSLATE = os.getenv("LOG_LEVEL_TRANSLATE", LOG_LEVEL).upper()

# Default Cache
OPA_CACHE_MAXSIZE = int(os.getenv("OPA_CACHE_MAXSIZE", "128"))
OPA_CACHE_TTL = int(os.getenv("OPA_CACHE_TTL", "120"))

# To disable caching define environment variable DISABLE_OPA_CACHE as true
# or set OPA_CACHE_MAXSIZE to 0 above, which will disable all
DISABLE_OPA_CACHE = os.getenv("DISABLE_OPA_CACHE", "False").lower() in (
    "true",
    "1",
    "t",
)
if DISABLE_OPA_CACHE:
    OPA_CACHE_MAXSIZE = 0

# Cache list of policies available, only cache latest result
OPA_LIST_CACHE_MAXSIZE = int(os.getenv("OPA_LIST_CACHE_MAXSIZE", "1"))
OPA_LIST_CACHE_TTL = int(os.getenv("OPA_LIST_CACHE_TTL", "60"))

# Cache policies themselves from OPA
OPA_FETCH_CACHE_MAXSIZE = int(os.getenv("OPA_FETCH_CACHE_MAXSIZE", OPA_CACHE_MAXSIZE))
OPA_FETCH_CACHE_TTL = int(os.getenv("OPA_FETCH_CACHE_TTL", "60"))

# Cache compile results from OPA (used by translate/search)
OPA_COMPILE_CACHE_MAXSIZE = int(
    os.getenv("OPA_COMPILE_CACHE_MAXSIZE", OPA_CACHE_MAXSIZE)
)
OPA_COMPILE_CACHE_TTL = int(os.getenv("OPA_COMPILE_CACHE_TTL", OPA_CACHE_TTL))

OPA_DATAAPI_CACHE_MAXSIZE = int(
    os.getenv("OPA_DATAAPI_CACHE_MAXSIZE", OPA_CACHE_MAXSIZE)
)
OPA_DATAAPI_CACHE_TTL = int(os.getenv("OPA_DATAAPI_CACHE_TTL", OPA_CACHE_TTL))

# RESPONSE_HEADERS_STRICT_TRANSPORT_SECURITY = "max-age=31536000; includeSubDomains"
# RESPONSE_HEADERS_CONTENT_SECURITY_POLICY = "frame-ancestors 'none'"
# RESPONSE_HEADERS_X_CONTENT_TYPE_OPTIONS = "nosniff"
# RESPONSE_HEADERS_CACHE_CONTROL = "no-store"
# RESPONSE_HEADERS_X_FRAME_OPTIONS = "DENY"

# MOCK ENTITLEMENT - turns off security - only used for development purposes. Do Not Use
MOCK_ENTITLEMENT = os.getenv("MOCK_ENTITLEMENT", "False").lower() in ("true", "1", "t")
MOCK_ENTITLEMENT_RESULT = {
    "desId": "test-user-with-access@testing.com",
    "memberEmail": "test-user-with-access@testing.com",
    "groups": [
        {
            "name": "service.plugin.user",
            "description": "Datalake Plugin-Manager users",
            "email": "service.plugin.user@osdu.example.com",
        },
        {
            "name": "service.csv-parser.admin",
            "description": "Datalake csv-parser admins",
            "email": "service.csv-parser.admin@osdu.example.com",
        },
        {
            "name": "service.secret.viewer",
            "description": "Viewer group for Secret service",
            "email": "service.secret.viewer@osdu.example.com",
        },
        {
            "name": "service.unit.admin",
            "description": "Datalake unit admins",
            "email": "service.unit.admin@osdu.example.com",
        },
        {
            "name": "service.edsdms.viewer",
            "description": "The viewer of the datalake edsdms service",
            "email": "service.edsdms.viewer@osdu.example.com",
        },
        {
            "name": "data.sdms.osdu.testtenant.1ee5e473-a6d7-46be-8e05-0e08dc97239a.viewer",
            "description": "seismic dms tenant osdu subproject testtenant viewer group",
            "email": "data.sdms.osdu.testtenant.1ee5e473-a6d7-46be-8e05-0e08dc97239a.viewer@osdu.example.com",
        },
        {
            "name": "service.binarydms.viewer",
            "description": "The viewer of the datalake binarydms service",
            "email": "service.binarydms.viewer@osdu.example.com",
        },
        {
            "name": "service.schema-service.viewers",
            "description": "Viewers group for ddms schema service",
            "email": "service.schema-service.viewers@osdu.example.com",
        },
        {
            "name": "service.legal.admin",
            "description": "Datalake Legal admins",
            "email": "service.legal.admin@osdu.example.com",
        },
        {
            "name": "users.data.root",
            "description": "Datalake root users",
            "email": "users.data.root@osdu.example.com",
        },
        {
            "name": "data.test1",
            "description": "Storage integration test group",
            "email": "data.test1@osdu.example.com",
        },
        {
            "name": "users.datalake.admins",
            "description": "Datalake admins",
            "email": "users.datalake.admins@osdu.example.com",
        },
        {
            "name": "service.file.editors",
            "description": "Editors group for file service",
            "email": "service.file.editors@osdu.example.com",
        },
        {
            "name": "service.dataset.viewers",
            "description": "Viewers group for Dataset service",
            "email": "service.dataset.viewers@osdu.example.com",
        },
        {
            "name": "users",
            "description": "Datalake users",
            "email": "users@osdu.example.com",
        },
        {
            "name": ADMIN_PERMISSION,
            "description": "Datalake policy admins",
            "email": "service.policy.admin@osdu.example.com",
        },
        {
            "name": "service.binarydms.admin",
            "description": "Datalake binarydms admins",
            "email": "service.binarydms.admin@osdu.example.com",
        },
        {
            "name": "service.schema-service.admin",
            "description": "Admin group for ddms schema service",
            "email": "service.schema-service.admin@osdu.example.com",
        },
        {
            "name": "service.storage.creator",
            "description": "Datalake Storage creators",
            "email": "service.storage.creator@osdu.example.com",
        },
        {
            "name": "service.file.viewers",
            "description": "Viewers group for file service",
            "email": "service.file.viewers@osdu.example.com",
        },
        {
            "name": "service.workflow.viewer",
            "description": "Viewers group for workflow service",
            "email": "service.workflow.viewer@osdu.example.com",
        },
        {
            "name": "data.sdms.osdu.autotest.2fb10b2e-c469-462a-be3a-6ba30cd31310.admin",
            "description": "seismic dms tenant osdu subproject autotest admin group",
            "email": "data.sdms.osdu.autotest.2fb10b2e-c469-462a-be3a-6ba30cd31310.admin@osdu.example.com",
        },
        {
            "name": "service.indexer.viewer",
            "description": "The viewer of the indexer service",
            "email": "service.indexer.viewer@osdu.example.com",
        },
        {
            "name": "service.legal.user",
            "description": "Datalake Legal users",
            "email": "service.legal.user@osdu.example.com",
        },
        {
            "name": "service.secret.admin",
            "description": "Admin group for Secret service",
            "email": "service.secret.admin@osdu.example.com",
        },
        {
            "name": "service.unit.creator",
            "description": "Datalake unit creators",
            "email": "service.unit.creator@osdu.example.com",
        },
        {
            "name": "cron.job",
            "description": "cron job role",
            "email": "cron.job@osdu.example.com",
        },
        {
            "name": "service.seismic-store.admin",
            "description": "Datalake seismic-store admins",
            "email": "service.seismic-store.admin@osdu.example.com",
        },
        {
            "name": "service.entitlements.admin",
            "description": "Datalake Entitlements admins",
            "email": "service.entitlements.admin@osdu.example.com",
        },
        {
            "name": "service.dataset.editors",
            "description": "Editors group for Dataset service",
            "email": "service.dataset.editors@osdu.example.com",
        },
        {
            "name": "service.workflow.creator",
            "description": "Creators group for workflow service",
            "email": "service.workflow.creator@osdu.example.com",
        },
        {
            "name": "service.search.user",
            "description": "Datalake Search users",
            "email": "service.search.user@osdu.example.com",
        },
        {
            "name": "service.unit.viewer",
            "description": "The viewer of the unit service",
            "email": "service.unit.viewer@osdu.example.com",
        },
        {
            "name": "data.default.owners",
            "description": "Default data owners",
            "email": "data.default.owners@osdu.example.com",
        },
        {
            "name": "data.sdms.osdu.testtenant.acdee207-acd7-47b1-bfc7-038736f05a08.admin",
            "description": "seismic dms tenant osdu subproject testtenant admin group",
            "email": "data.sdms.osdu.testtenant.acdee207-acd7-47b1-bfc7-038736f05a08.admin@osdu.example.com",
        },
        {
            "name": "service.workflow.admin",
            "description": "Admin group for workflow service",
            "email": "service.workflow.admin@osdu.example.com",
        },
        {
            "name": "service.entitlements.user",
            "description": "Datalake Entitlements users",
            "email": "service.entitlements.user@osdu.example.com",
        },
        {
            "name": "service.edsdms.user",
            "description": "Datalake edsdms user",
            "email": "service.edsdms.user@osdu.example.com",
        },
        {
            "name": USER_PERMISSION,
            "description": "The viewer of the datalake policy service",
            "email": "service.policy.user@osdu.example.com",
        },
        {
            "name": "service.ingest.viewer",
            "description": "The viewer of the ingest service",
            "email": "service.ingest.viewer@osdu.example.com",
        },
        {
            "name": "service.ingest.admin",
            "description": "Datalake ingest admins",
            "email": "service.ingest.admin@osdu.example.com",
        },
        {
            "name": "service.indexer.admin",
            "description": "Datalake indexer admins",
            "email": "service.indexer.admin@osdu.example.com",
        },
        {
            "name": "service.schema-service.editors",
            "description": "Editors group for ddms schema service",
            "email": "service.schema-service.editors@osdu.example.com",
        },
        {
            "name": "service.ingest.creator",
            "description": "Datalake ingest creators",
            "email": "service.ingest.creator@osdu.example.com",
        },
        {
            "name": "service.secret.editor",
            "description": "Editor group for Secret service",
            "email": "service.secret.editor@osdu.example.com",
        },
        {
            "name": "users.datalake.viewers",
            "description": "Datalake viewers",
            "email": "users.datalake.viewers@osdu.example.com",
        },
        {
            "name": "data.default.viewers",
            "description": "Default data viewers",
            "email": "data.default.viewers@osdu.example.com",
        },
        {
            "name": "service.search.admin",
            "description": "Datalake Search admins",
            "email": "service.search.admin@osdu.example.com",
        },
        {
            "name": "service.seismic-store.creator",
            "description": "Datalake seismic-store creators",
            "email": "service.seismic-store.creator@osdu.example.com",
        },
        {
            "name": "service.seismic-store.viewer",
            "description": "The viewer of the seismic-store service",
            "email": "service.seismic-store.viewer@osdu.example.com",
        },
        {
            "name": "users.datalake.editors",
            "description": "Datalake editors",
            "email": "users.datalake.editors@osdu.example.com",
        },
        {
            "name": "data.integration.test",
            "description": "Storage integration test group",
            "email": "data.integration.test@osdu.example.com",
        },
        {
            "name": "service.legal.editor",
            "description": "Datalake Legal editors",
            "email": "service.legal.editor@osdu.example.com",
        },
        {
            "name": "service.edsdms.admin",
            "description": "Datalake edsdms admins",
            "email": "service.edsdms.admin@osdu.example.com",
        },
        {
            "name": "data.sdms.osdu.autotest.73e1d8ab-c9b6-472c-ac5a-694e82e12bf8.viewer",
            "description": "seismic dms tenant osdu subproject autotest viewer group",
            "email": "data.sdms.osdu.autotest.73e1d8ab-c9b6-472c-ac5a-694e82e12bf8.viewer@osdu.example.com",
        },
        {
            "name": "service.messaging.user",
            "description": "Datalake Messaging users",
            "email": "service.messaging.user@osdu.example.com",
        },
        {
            "name": "service.csv-parser.creator",
            "description": "Datalake csv-parser creators",
            "email": "service.csv-parser.creator@osdu.example.com",
        },
        {
            "name": "service.delivery.viewer",
            "description": "Delivery role",
            "email": "service.delivery.viewer@osdu.example.com",
        },
        {
            "name": "service.storage.viewer",
            "description": "The viewer of the datalake storage service",
            "email": "service.storage.viewer@osdu.example.com",
        },
        {
            "name": "service.edsdms.creator",
            "description": "Datalake edsdms creators",
            "email": "service.edsdms.creator@osdu.example.com",
        },
        {
            "name": "service.storage.admin",
            "description": "Datalake Storage admins",
            "email": "service.storage.admin@osdu.example.com",
        },
        {
            "name": "service.binarydms.creator",
            "description": "Datalake binarydms creators",
            "email": "service.binarydms.creator@osdu.example.com",
        },
        {
            "name": "service.indexer.creator",
            "description": "Datalake indexer creators",
            "email": "service.indexer.creator@osdu.example.com",
        },
        {
            "name": "service.csv-parser.viewer",
            "description": "The viewer of the datalake csv-parser service",
            "email": "service.csv-parser.viewer@osdu.example.com",
        },
        {
            "name": "users.datalake.ops",
            "description": "Datalake ops",
            "email": "users.datalake.ops@osdu.example.com",
        },
    ],
}

# Bootstrap
# Allow bootstrap API force=True to replace existing bundle with default
# This is useful:
# in automated testing to reset/start from a known state
# if someone breaks OPA by saving rego with syntax errors and it's not know which rego caused the issue
ALLOW_FORCE_BOOTSTRAP = os.getenv("ALLOW_FORCE_BOOTSTRAP", "True").lower() in (
    "true",
    "1",
    "t",
)

# Check policies before saving them to bundles
ENABLE_VERIFY_POLICY = os.getenv("ENABLE_VERIFY_POLICY", "True").lower() in (
    "true",
    "1",
    "t",
)

# Allow/Disable config API
ALLOW_CONFIG_API = os.getenv("ALLOW_CONFIG_API", "True").lower() in ("true", "1", "t")

# Mask config details from /config API
MASK_OPA_CONFIG_DETAILS = os.getenv("MASK_OPA_CONFIG_DETAILS", "False").lower() in (
    "true",
    "1",
    "t",
)

MASK_CONFIG_ENV = os.getenv("MASK_CONFIG_ENV", "True").lower() in ("true", "1", "t")

ENVOY = os.getenv("ZIPKIN", "True").lower() in ("true", "1", "t")
NAMESPACE_PATH = "/var/run/secrets/kubernetes.io/serviceaccount/namespace"

# Kubernetes details
if "NAMESPACE" in os.environ:
    NAMESPACE = os.getenv("NAMESPACE")
elif "AWS_SERVICE_NAMESPACE" in os.environ:
    NAMESPACE = os.getenv("AWS_SERVICE_NAMESPACE") + "-core"
elif os.path.isfile(NAMESPACE_PATH):
    with open(NAMESPACE_PATH) as f:
        NAMESPACE = f.read()
else:
    NAMESPACE = "osdu-services"

# Kubernetes ServiceAccount bearer token
TOKEN = None
if os.path.isfile("/var/run/secrets/kubernetes.io/serviceaccount/token"):
    with open(NAMESPACE_PATH) as f:
        TOKEN = f.read()

OPA_CONFIG_MAP = os.getenv("OPA_CONFIG_MAP", "opa-config")
OPA_MAX_POLLING = 3600

ALLOW_DATA_PARTITION_ID_PATTERN = "^[A-Za-z0-9_-]*$"
ALLOW_CORRELATION_ID_PATTERN = "^[A-Za-z0-9_-]*$"

# OCI Registry not supported
OPA_SUPPORTED_SERVICES = ["s3", "gcs", "gcp", "blob", "nginx"]

CLOUD_PROVIDER = os.getenv("CLOUD_PROVIDER", "LOCAL")
